/*
 * Last update: 2005-01-11
 * Created ...: 2004-12-06
 * @author  jmedved <jmedved@jmedved.com>
 */

package medo.dotnet;

/*** Class for managing Date information in .NET way with small adjustments */
public class DateTime {
    
    private java.util.Calendar xCalendar = java.util.Calendar.getInstance();

    /** Initializes class with current date and time
     */
    public DateTime() {
        xCalendar.setTime(new java.util.Date());
    }
    /**
     * @param date java.util.Date used to initialize class
     */
    public DateTime(java.util.Date date) {
        xCalendar.setTime(date);
    }
    /**
     * @param calendar java.util.Calendar used to initialize class
     */
    public DateTime(java.util.Calendar calendar) {
        xCalendar.setTime(calendar.getTime());
    }
    /**
     * @param year desired year value
     * @param month desired month value
     * @param day desired day value
     */        
    public DateTime(int year, int month, int day) {
        xCalendar.set(year,month-1,day,0,0,0);
        xCalendar.set(java.util.Calendar.MILLISECOND, 0);
    }
    /**
     * @param year desired year value
     * @param month desired month value
     * @param day desired day value
     * @param hour desired hour value
     * @param minute desired minute value
     * @param second desired second value
     */        
    public DateTime(int year, int month, int day, int hour, int minute, int second) {
        xCalendar.set(year,month-1,day,hour,minute,second);
        xCalendar.set(java.util.Calendar.MILLISECOND, 0);
    }
    /**
     * @param year desired year value
     * @param month desired month value
     * @param day desired day value
     * @param hour desired hour value
     * @param minute desired minute value
     * @param second desired second value
     * @param millisecond desired millisecond value
     */
    public DateTime(int year, int month, int day, int hour, int minute, int second, int millisecond) {
        xCalendar.set(year,month-1,day,hour,minute,second);
        xCalendar.set(java.util.Calendar.MILLISECOND, millisecond);
    }

    /**
     * @return String reprezentation of date value
     */
    public String toString() {
        return xCalendar.getTime().toString();
    }

    /**
     * @return String reprezentation of date value
     */
    public String toShortDateString() {
        return java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT).format(xCalendar.getTime());
    }

    /**
     * @return String reprezentation of date value
     */
    public String toLongDateString() {
        return java.text.DateFormat.getDateInstance(java.text.DateFormat.LONG).format(xCalendar.getTime());
    }

    /**
     * @return String reprezentation of date value
     */
    public String toShortTimeString() {
        return java.text.DateFormat.getTimeInstance(java.text.DateFormat.SHORT).format(xCalendar.getTime());
    }

    /**
     * @return String reprezentation of date value
     */
    public String toLongTimeString() {
        return java.text.DateFormat.getTimeInstance(java.text.DateFormat.LONG).format(xCalendar.getTime());
    }


    /**
     * @return java.util.Date object
     */
    public java.util.Date toDate() {
        return xCalendar.getTime();
    }
    
    /**
     * @param days number of days to add
     * @return new DateTime object
     */        
    public DateTime AddDays(int days) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.DAY_OF_YEAR, days);
        return new DateTime(newCal);
    }

    /**
     * @param hours number of hours to add
     * @return new DateTime object
     */        
    public DateTime AddHours(int hours) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.HOUR, hours);
        return new DateTime(newCal);
    }

    /**
     * @param milliseconds number of milliseconds to add
     * @return new DateTime object
     */        
    public DateTime AddMilliseconds(int milliseconds) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.MILLISECOND, milliseconds);
        return new DateTime(newCal);
    }

    /**
     * @param minutes number of minutes to add
     * @return new DateTime object
     */        
    public DateTime AddMinutes(int minutes) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.MINUTE, minutes);
        return new DateTime(newCal);
    }

    /**
     * @param months number of months to add
     * @return new DateTime object
     */        
    public DateTime AddMonths(int months) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.MONTH, months);
        return new DateTime(newCal);
    }

    /**
     * @param seconds number of seconds to add
     * @return new DateTime object
     */
    public DateTime AddSeconds(int seconds) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.SECOND, seconds);
        return new DateTime(newCal);
    }

    /** 
     * @param years number of years to add
     * @return new DateTime object
     */
    public DateTime AddYears(int years) {
        java.util.Calendar newCal = java.util.Calendar.getInstance();
        newCal.setTime(xCalendar.getTime());
        newCal.add(java.util.Calendar.YEAR, years);
        return new DateTime(newCal);
    }

    /**
     * @return month day 
     */        
    public int getDay() {
        return xCalendar.get(java.util.Calendar.DAY_OF_MONTH);
    }

    /**
     * @return week day
     */        
    public int getDayOfWeek() {
        return xCalendar.get(java.util.Calendar.DAY_OF_WEEK);
    }

    /**
     * @return year day
     */        
    public int getDayOfYear() {
        return xCalendar.get(java.util.Calendar.DAY_OF_YEAR);
    }

    /**
     * @return hour
     */        
    public int getHour() {
        return xCalendar.get(java.util.Calendar.HOUR);
    }

    /**
     * @return minute
     */        
    public int getMinute() {
        return xCalendar.get(java.util.Calendar.MINUTE);
    }

    /**
     * @return millisecond
     */        
    public int getMillicecond() {
        return xCalendar.get(java.util.Calendar.MILLISECOND);
    }

    /**
     * @return month (1-12)
     */        
    public int getMonth() {
        return xCalendar.get(java.util.Calendar.MONTH)+1;
    }

    /**
     * @return second
     */        
    public int getSecond() {
        return xCalendar.get(java.util.Calendar.SECOND);
    }

    /**
     * @return year
     */        
    public int getYear() {
        return xCalendar.get(java.util.Calendar.YEAR);
    }

    /** Static function that returns current date and time component only
     * @return current date and time
     */        
    public static DateTime getNow() {
        return (new DateTime(new java.util.Date()));
    }

    /** Static function that returns current date component only
     * @return current date
     */        
    public static DateTime getToday() {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        cal.setTime(new java.util.Date());
        cal.set(java.util.Calendar.HOUR, 0);
        cal.set(java.util.Calendar.MINUTE, 0);
        cal.set(java.util.Calendar.SECOND, 0);
        cal.set(java.util.Calendar.MILLISECOND, 0);
        return new DateTime(cal);
    }

    /** Static function that returns current time component only
     * @return current time
     */        
    public static DateTime getTimeOfDay() {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        cal.setTime(new java.util.Date());
        cal.set(java.util.Calendar.YEAR, 0);
        cal.set(java.util.Calendar.MONTH, 0);
        cal.set(java.util.Calendar.DAY_OF_MONTH, 0);
        return new DateTime(cal);
    }

    /**
     * @return number of days in current month
     */
    public int getDaysInMonth() {
        java.util.Calendar cal = (java.util.Calendar)xCalendar.clone();
        cal.add(java.util.Calendar.MONTH,1);
        cal.set(java.util.Calendar.DAY_OF_MONTH,1);
        cal.add(java.util.Calendar.DAY_OF_YEAR,-1);
        return cal.get(java.util.Calendar.DAY_OF_MONTH);
    }

    /** Static function that returns number of days in months for desired year and month
     * @return number of days
     */        
    public static int getDaysInMonth(int year, int month) {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        cal.set(year,month-1,1,0,0,0);
        cal.add(java.util.Calendar.MONTH,1);
        cal.set(java.util.Calendar.DAY_OF_MONTH,1);
        cal.add(java.util.Calendar.DAY_OF_YEAR,-1);
        return cal.get(java.util.Calendar.DAY_OF_MONTH);
    }

    /**
     * @return is current year a lap year
     */        
    public boolean getIsLeapYear() {
        java.util.Calendar cal = (java.util.Calendar)xCalendar.clone();
        cal.set(java.util.Calendar.MONTH,11);
        cal.set(java.util.Calendar.DAY_OF_MONTH,31);
        if (cal.get(java.util.Calendar.DAY_OF_YEAR)==366) {
            return true;
        } else {
            return false;
        }
    }

    /** Static function that returns is lap year for desired year
     * @return is year a lap year
     */        
    public static boolean getIsLeapYear(int year) {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        cal.set(year,0,1,0,0,0);
        cal.set(java.util.Calendar.MONTH,11);
        cal.set(java.util.Calendar.DAY_OF_MONTH,31);
        if (cal.get(java.util.Calendar.DAY_OF_YEAR)==366) {
            return true;
        } else {
            return false;
        }
    }

}
